@@ -60,7 +60,7 @@ def factory(producer: Callable[[], Factory.Produces]) -> Factory.Produces:
 class DataClassMeta(type):
     """The metaclass that implements data class behaviour."""
     DEFAULT_OPTIONS = dict(init=True, repr=True, eq=True, order=False, unsafe_hash=False, frozen=False,
-                           hide_internals=True, iter=False, kwargs=False, slots=False)
+                           kw_only=False, match_args=True, hide_internals=True, iter=False, kwargs=False, slots=False)
 
     def __new__(mcs, name, bases, dict_, **kwargs):
         """Create a new data class."""
@@ -134,10 +134,13 @@ class DataClassMeta(type):
         if (options['eq'] and options['frozen']) or options['unsafe_hash']:
             '__hash__' in all_attrs or dict_.setdefault('__hash__', generate_hash(all_annotations))
 
+        if options['match_args']:
+            dict_.setdefault('__match_args__', tuple(sorted(all_annotations, key=all_defaults.__contains__)))
+
         return super().__new__(mcs, name, bases, dict_)
 
-    # noinspection PyMissingConstructor,PyUnresolvedReferences,PyTypeChecker,PyUnusedLocal
-    def __init__(cls, *args, **kwargs):
+    # noinspection PyMissingConstructor,PyUnresolvedReferences,PyTypeChecker
+    def __init__(cls, *_, **__):
         if cls.__dataclass__['eq'] and cls.__dataclass__['order']:
             from functools import total_ordering
             total_ordering(cls)
@@ -167,33 +170,33 @@ def is_user_func(obj: Any, object_methods=frozenset(vars(object).values())) -> b
 
 def generate_init(annotations: Dict, defaults: Dict, options: Dict, user_init: bool) -> Function:
     """Generate and return an __init__ method for a data class. This method has as parameters all fields of the data
-    class. When the data class is initialised, arguments to this function are applied to the fields of the new instance.
-    A user-defined __init__, if present, must be aliased to avoid conflicting."""
+    class. When the data class is initialised, arguments to this function are applied to the fields of the new
+    instance."""
+    kw_only = ['*'] if options['kw_only'] else []
     arguments = [a for a in annotations if a not in defaults]
     default_arguments = [f'{a}={a}' for a in defaults]
-    args = ['*args'] if user_init else []
+    args = ['*args'] if user_init and not kw_only else []
     kwargs = ['**kwargs'] if user_init or options['kwargs'] else []
 
-    parameters = ', '.join(arguments + default_arguments + args + kwargs)
+    parameters = ', '.join(kw_only + arguments + default_arguments + args + kwargs)
 
     # surprisingly, given global lookups are slow, using them is the fastest way to compare a field to its default
     # the alternatives are to look up on self (which wouldn't work when slots=True) or look up self.__defaults__
-    default_names = {f'default_{n}': v for n, v in defaults.items()}
+    copied = {n: (f'd_{n}', v) for n, v in defaults.items() if hasattr(v, 'copy')}
 
     # determine what to do with arguments before assignment. If the argument matches a mutable default, make a copy
-    references = {n: f'{n}.copy() if {n} is default_{n} else {n}' if n in defaults and hasattr(defaults[n], 'copy')
-                  else n for n in annotations}
+    references = {n: f'{n}.copy() if {n} is {d} else {n}' for n, (d, _) in copied.items()}
 
     # if the class is frozen, use the necessary but far slower object.__setattr__
-    assignments = [f'object.__setattr__(self, {n!r}, {r})' if options['frozen']
-                   else f'self.{n} = {r}' for n, r in references.items()]
+    assigner = 'object.__setattr__(self, {!r}, {})' if options['frozen'] else 'self.{} = {}'
+    assignments = [assigner.format(n, references.get(n, n)) for n in annotations]
+
+    # if defined, call __post_init__ with leftover arguments
+    call_post_init = f'self.__post_init__({", ".join(args + kwargs)})' if user_init else ''
 
     # generate the function
-    lines = [f'def __init__(self, {parameters}):',
-             *assignments,
-             'self.__post_init__(*args, **kwargs)' if user_init else '']
-
-    return eval_function('__init__', lines, annotations, defaults, default_names)
+    lines = [f'def __init__(self, {parameters}):', *assignments, call_post_init]
+    return eval_function('__init__', lines, annotations, defaults, dict(copied.values()))
 
 
 def generate_hash(annotations: Dict[str, Type]) -> Function:
@@ -228,6 +231,5 @@ def __repr__(self):
     return f'{type(self).__name__}({field_values})'
 
 
-# noinspection PyUnusedLocal
-def __setattr__(self, *args):
+def __setattr__(*_):
     raise AttributeError('Frozen class')
