@@ -18,13 +18,17 @@ def dataclass(cls: Optional[type] = None, *, meta=DataClassMeta, **options) -> T
     :key init: Generate an __init__ method
     :key repr: Generate a __repr__ method
     :key eq: Generate an __eq__ method
+    :key frozen: Allow field reassignment after initialisation
     :key order: Generate comparison methods other than __eq__
     :key unsafe_hash: Force generation of __hash__
-    :key frozen: Allow field reassignment after initialisation
-    :key hide_internals: Hide internal methods in __repr__, __iter__ and comparison methods
-    :key iter: Generate an __iter__ method
+    :key match_args: Generate a __match_args__ attribute
+    :key kw_only: Make all parameters to the generated __init__ keyword-only
+
     :key kwargs: Append **kwargs to the list of initialiser parameters
     :key slots: Generate __slots__
+    :key iter: Generate an __iter__ method
+    :key hide_internals: Hide internal methods in __repr__
+
     :return: The newly created data class
     """
     assert issubclass(meta, DataClassMeta)
